{# device_type: arduino101 #}
{% extends 'base.jinja2' %}
{% block body %}
board_id: '{{ board_id|default('0000000000') }}'
usb_vendor_id: '8087'
usb_product_id: '0aba'

actions:
  deploy:
    methods:
      image:
        parameters:

  boot:
    connections:
      serial:
    methods:
      dfu:
        parameters:
          command:
            dfu-util
{% endblock body -%}
