# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-18 09:56
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0028_notify_callback")]

    operations = [migrations.RemoveField(model_name="testjob", name="_results_bundle")]
