# Common Amlogic Meson GX configuration
{% extends 'base-uboot.jinja2' %}

{% set uboot_mkimage_arch = 'arm64' %}
{% set console_device = console_device|default('ttyAML0') %}
{% set baud_rate = baud_rate|default(115200) %}
{% set interrupt_prompt = interrupt_prompt|default('stop autoboot') %}
{% set interrupt_ctrl_list = ['c'] %}

{% set bootm_kernel_addr = '0x1080000' %}
{% set bootm_dtb_addr = '0x1000000' %}
{% set bootm_ramdisk_addr = '0x3000000' %}
