{% extends "layouts/content.html" %}
{% load i18n %}

{% block title %}{% trans "Password reset" %}{% endblock %}

{% block content %}
<div class="row">
  <div class="col-md-12">
    <p>{% trans "Forgotten your password? Enter your email address below, and we'll email instructions for setting a new one." %}</p>
    <form class="form-horizontal" action="." method="post">
      {% csrf_token %}
      <div class="form-group{% if form.email.errors %} has-error{% endif %}">
        <label class="col-md-4 control-label" for="id_email">{% trans "Email address:" %}</label>
        <div class="col-md-4">
          {{ form.email }}
          {% for error in form.email.errors %}
          <p class="help-block">{{ error }}</p>
          {% endfor %}
        </div>
      </div>
      <div class="form-group">
        <div class="col-md-8 col-md-offset-4">
          <button type="submit" class="btn btn-primary">{% trans "Reset my password" %}</button>
        </div>
      </div>
    </form>
  </div>
</div>
{% endblock %}
