{% extends 'imx7s-warp.jinja2' %}
{% set hard_reset_command = ['/usr/local/lab-scripts/usb_hub_control --usb_port 15 --mode off --port 7501', 'sleep 5', '/usr/local/lab-scripts/usb_hub_control --usb_port 15 --mode sync --port 7501'] %}
{% set power_off_command = '/usr/local/lab-scripts/usb_hub_control --usb_port 15 --mode off --port 7501' %}
{% set power_on_command = '/usr/local/lab-scripts/usb_hub_control --usb_port 15 --mode sync --port 7501' %}
{% set connection_command = 'telnet serial4 7006' %}
{% set uboot_mass_storage_device = '/dev/disk/by-id/usb-Linux_UMS_disk_0_WaRP7-0x742400d3000000e6-0:0' %}
