{# device_type = 'rk3399-gru-kevin' #}
{% extends 'base-depthcharge.jinja2' %}

{% set mkimage_fit_arch = 'arm64' %}
{% set start_message = start_message | default('Starting depthcharge on gru...') %}
{% set bootloader_prompt = bootloader_prompt | default('gru:') %}
{% set console_device = console_device | default('ttyS2') %}
{% set extra_kernel_args = base_kernel_args | default('earlyprintk=ttyS2,115200n8 console=tty1') %}
