{% extends 'base.jinja2' %}
{% block body %}

actions:
  deploy:
    connections:
      serial:
    methods:
      ssh:
        options:
{{ ssh_options }}
        # primary connections get this from the device dictionary.
        # secondary connections get this from the lava-multinode protocol support.
        host: '{{ ssh_host|default('') }}'
        port: {{ ssh_port|default(22) }}
        identity_file: {{ ssh_identity_file }}

  boot:
    connections:
      serial:
    methods:
# can be removed by defining this block empty in the device config
{% block schroot %}
      schroot:
        command:
{{ schroot }}
        name:
          {{ schroot_name_list|default(['unstable', 'sid'])}}
{% endblock schroot %}
      ssh:

{% endblock body %}
