{# device_type = "dra7-evm" #}
{% extends 'base-uboot.jinja2' %}
{% set console_device = console_device | default('ttyO0') %}
{% set baud_rate = baud_rate|default(115200) %}
{% set bootm_kernel_addr = '0x82000000' %}
{% set bootm_ramdisk_addr = '0x83000000' %}
{% set bootm_dtb_addr = '0x815f0000' %}
{% set bootz_kernel_addr = '0x82000000' %}
{% set bootz_ramdisk_addr = '0x83000000' %}
{% set bootz_dtb_addr = '0x88000000' %}
{% set uboot_mkimage_arch = 'arm' %}
