{% extends 'beaglebone-black.jinja2' %}
{% set usb_uuid = 'usb-SanDisk_Ultra_20060775320F43006019-0:0' %}
{% set connection_commands = {'uart0': "telnet localhost 6000"} %}
{% set connection_tags = {'uart0': ['primary', 'telnet']} %}
{% set connection_list = ['uart0'] %}
{% set hard_reset_command = (
"/usr/bin/pduclient --daemon localhost --hostname pdu --command reboot --port 08") %}
{% set power_off_command = (
"/usr/bin/pduclient --daemon localhost --hostname pdu --command off --port 08") %}
{% set power_on_command = (
"/usr/bin/pduclient --daemon localhost --hostname pdu --command on --port 08") %}
