{% extends 'mps.jinja2' %}

{# Primary connection must be something LAVA knows how to disconnect from, e.g. telnet #}
{# This example is not yet handled (minicom not supported) #}
{# If disconnecting from minicom is handled in the future this may work (but command may need altering #}
{% set connection_list = ['uart0'] %}
{% set connection_commands = {'uart0': 'minicom -b 115200 /dev/ttyUSB0'} %}
{% set connection_tags = {'uart0': ['primary', 'minicom']} %}

{% set power_on_command = '/opt/lava-lab/shared/lab-scripts/snmp_pdu_control --hostname 192.168.0.23 --port 1 --delay 20 --command on' %}
{% set power_off_command = '/opt/lava-lab/shared/lab-scripts/snmp_pdu_control --hostname 192.168.0.23 --port 1 --delay 20 --command off' %}
{% set hard_reset_command = '/opt/lava-lab/shared/lab-scripts/snmp_pdu_control --hostname 192.168.0.23 --port 1 --delay 20 --command reboot' %}

{% set usb_filesystem_label = 'V2M-MPS' %}