{# device_type: adb-nuc #}
{% extends 'base.jinja2' %}
{% block body %}

actions:
  deploy:
    connections:
      lxc:
    methods:
      lxc:
  boot:
    connections:
      lxc:
    methods:
      lxc:
{% endblock body %}
