{% extends 'base-grub.jinja2' %}
{% set boot_character_delay = boot_character_delay | default(100) %}
{% set efi_grub = True %}
{% set console_device = "ttyS0" %}
{% set baud_rate = 9600 %}
{% set has_dtb = True %}
