@import url("../../admin/css/widgets.css");

div.selector { clear: both; }
div.selector span.helptext { display: none; }
div.selector h2 { margin: 0; font-size: 11pt; }
div.selector a { text-decoration: none; }
div.selector select { height: 10em; }
div.selector ul.selector-chooser { margin-top: 5.5em; }
div.selector .selector-chosen select {
  border: 1px solid rgb(204, 204, 204);
  border-top: none;
}

.list-container {
    border: 1px solid #000000;
    clear: both;
    margin: 10px 10px 10px 10px;
    padding: 10px;
    width: 50%;
}

.form-field {
    margin-bottom: 5px;
    vertical-align: top;
}

.form-field label {
    vertical-align: top;
    width: 120px;
    display: inline-block;
    margin-left: 10px;
}

.submit-button {
    margin-top: 20px;
    margin-left: 10px;
}

.filter-headline {
    font-weight: bold;
    font-size: 16px;
}

.filter-container {
    margin-bottom: 10px;
    clear: both;
}

.filter-title {
    font-weight: bold;
    font-size: 15px;
    margin-bottom: 10px;
}

.chart-title {
    font-weight: bold;
    font-size: 15px;
    margin-bottom: 10px;
}

.chart-title-link {
    float: right;
    margin-left: 20px;
    font-size: 13px;
}

.errors {
    color: red;
}

.fields-container {
    margin-left: 10px;
}

#filters_div {
    margin: 10px 0 0 10px;
    border: 1px solid #000000;
    clear: both;
    width: 75%;
    padding: 5px 0 10px 10px;
    overflow: auto;
}

#alias_container {
    font-weight: bold;
    float: left;
    display: none;
}

#test_select_container {
    margin-bottom: 10px;
}

.outer-chart {
}

.inner-chart {
    width: 82%;
    display: inline-block;
}

.legend {
    width: 15%;
    display: inline-block;
    overflow-x: scroll;
    white-space: nowrap;
}

.legendLabel {
    text-decoration: underline;
    padding-left: 2px;
}

.legend tr {
    cursor: pointer;
}

#main_container {
    padding-left: 20px;
    margin-bottom: 20px;
}

.headline-container {
    margin: 20px 0 10px 0px;
    width: 80%;
}

.dates-container {
    margin: 10px 0 10px 0px;
    width: 80%;
}

.data-table-modal {
    width: 1000px;
}

.table-link-container {
    margin-left: 30px;
    font-size: 16px;
}

.toggle-options-container {
    margin: 10px 0 10px 0px;
    width: 80%;
}

.toggle-options-container img {
    border: 0px;
    float: right;
    font-size: 12px;
    margin-top: 3px;
}

.toggle-legend {
    margin-left: 30px;
}

.toggle-delta {
    margin-left: 30px;
}

.toggle-checkbox {
    vertical-align: top;
    margin-left: 5px;
}

.chart-headline {
    font-size: 17px;
    font-weight: bold;
    color: #000;
    margin-right: 10px;
}

.sort {
    border: 1px solid #000000;
    width: 100%;
    margin: 10px 0 10px 0;
    padding: 7px;
    height: 180px;
}

.sort:hover {
    cursor: row-resize;
}

.print-menu {
    position: absolute;
    font-size: 12px;
    z-index: 1;
}

.print-menu .print-menu-item {
	margin:0;
	padding: 0;
	zoom: 1;
	float: left;
	clear: left;
	width: 100%;
}

.print-menu .print-menu-item a {
	text-decoration:none;
	display:block;
	padding:.2em .4em;
	line-height:1.5;
	zoom:1;
}

.print-menu .print-menu-item a.print-state-hover,
.print-menu .print-menu-item a.print-state-active {
	font-weight: normal;
	margin: -1px;
	background: #5f83b9;
	color: #FFFFFF;
	text-shadow: 0px 1px 1px #234386;
	border-color: #466086;
	-moz-border-radius: 0; -webkit-border-radius: 0; border-radius: 0;
}

#outer-table, .inner-table {
    border-collapse: collapse;
    color: black;
}

.inner-table > tbody > tbody > tr > td > a {
    color: black;
}

.inner-table > tbody > tbody > tr > td > span > a {
    color: black;
}

#outer-table > tbody > tr > td {
    padding: 0;
    vertical-align: top;
}

#outer-table th {
    text-align: left;
}

.inner-table-names td, .inner-table-names th {
    width: auto;
    white-space: nowrap;
    padding: 3px 4px;
    border: thin solid black;
}

.inner-table td, .inner-table th {
    min-width: 105px;
    padding: 3px 4px;
    border: thin solid black;
}

.inner-table td.present a {
    color: #00a;
    text-decoration: none;
}

.inner-table td.present a:hover {
    text-decoration: underline;
}

.inner-table td.pass {
    background-color: #4e4;
}

.inner-table td.fail {
    background-color: #e44;
}

.inner-table td.missing {
    background-color: #bbb;
}

#tooltip {
    position: absolute;
    z-index: 3000;
    border: 1px solid #111;
    background-color: #eee;
    color: #000;
    padding: 5px;
    opacity: 0.85;
}

#tooltip h3, #tooltip div { margin: 0; }

.scroller {
    width: 800px;
    overflow-x: scroll;
    overflow-y: visible;
}

#scroller td:first-child, #scroller th:first-child {
    border-left: none;
}

.bug-link-container {
    overflow: hidden;
    float: right;
}

#test_edit_dialog {
    background-color: white;
}

.dialog-title {
    font-size: 20px;
    font-weight: bold;
    margin-bottom: 10px;
}

.edit-dialog {
    background-color: #eee;
}

.dialog-field {
    margin-bottom: 5px;
    font-weight: bold;
    vertical-align: top;
}

.dialog-field input {
    margin-left: 35px;
}

.dialog-field select {
    margin-left: 10px;
    height: 250px;
}

.dialog-submit {
    margin-top: 5px;
}

.dialog-input {
    width: 300px;
}

.dialog-select {
    width: 300px;
}

.dialog-field-title {
    vertical-align: top;
}

.group-link {
    margin: 0 0 0 10px;
    float: left;
}

.zoom-out-button {
    position: absolute;
    font-size: smaller;
    color: #999;
    background-color: #eee;
    padding: 3px;
    width: 57px;
    height: 20px;
    margin-top: 13px;
    z-index: 1;
    cursor: pointer;
    display: inline;
}

.form-field, #condition_form p {
    margin-bottom: 5px;
    vertical-align: top;
}

.form-field label, #condition_form p label {
    vertical-align: top;
    width: 120px;
    display: inline-block;
    margin-left: 10px;
}

.group-link {
    margin: 0 0 0 10px;
    float: left;
}


.loading_dialog {
    height: 45px;
    width: 250px;
}

.help-text {
    font-size: 11px;
}