{# device_type: frdm-k64f #}
{% extends 'base.jinja2' %}
{% block body %}
board_id: '{{ board_id|default('0000000000') }}'
usb_vendor_id: '0d28'
usb_product_id: '0204'

actions:
  deploy:
    connections:
      lxc:
    methods:
      lxc:
      image:
        parameters:

  boot:
    connections:
      serial:
      lxc:
      ssh:
    methods:
      lxc:
      pyocd:
        parameters:
          command:
            pyocd-flashtool
          options:
          - -d {{ debug|default('debug') }}
          - -t k64f
          - -f 3000000
      cmsis-dap:
        parameters:
          usb_mass_device: '{{ usb_mass_device|default('/notset') }}'
{% endblock body -%}
