{# device_type = "beagle-xm" #}
{% extends 'base-uboot.jinja2' %}

{% set console_device = console_device|default('ttyO2') %}
{% set baud_rate = baud_rate|default(115200) %}
{% set bootloader_prompt = bootloader_prompt|default('=>') %}
{% set bootm_kernel_addr = '0x80000000' %}
{% set bootm_ramdisk_addr = '0x81600000' %}
{% set bootm_dtb_addr = '0x815f0000' %}
{% set bootz_kernel_addr = '0x81000000' %}
{% set bootz_ramdisk_addr = '0x82000000' %}
{% set bootz_dtb_addr = '0x81f00000' %}
{% set uboot_mkimage_arch = 'arm' %}
{% set uboot_needs_usb = true %}
{% set uboot_usb_commands = '          - "setenv usbethaddr 00:40:5c:26:0a:6c; usb start"' %}
