{# set device_type = "hi6220-hikey-bl - based on r2 based on 960" #}
{% extends 'base-fastboot.jinja2' %}
{% set boot_character_delay = boot_character_delay | default(10) %}
{% set root_device = root_device | default('/dev/mmcblk0p9') %}
{% set base_kernel_args = base_kernel_args|default('') %}
{% set console_device = console_device|default('ttyAMA3') %}
{% set baud_rate = baud_rate|default('115200') %}
{% set fastboot_sequence = ['boot'] %}
{% set recovery_mode_command = recovery_mode_command|default('') %}
{% set fastboot_only_command = fastboot_only_command|default('') %}
{% set fastboot_interrupt_params = "
        interrupt_prompt: 'Android Fastboot mode'
        interrupt_string: ' '"%}
{% set flash_cmds_order = ['ptable', 'xloader', 'fastboot', 'nvme', 'fw_lpm3',
'trustfirmware', 'boot', 'dts', 'system', 'userdata', 'cache'] %}
{% set fastboot_boot_grub = "
      grub:
        reset_device: False
        sequence:
        - wait-fastboot-interrupt
        installed:
          commands:
            - boot
"%}
{# Different device-types will have different types and numbers of commands. #}
{% set recovery_mode = "
      recovery:
        commands:
"%}
