{# device_type: 'mps' #}
{% extends 'base.jinja2' %}
{% block body %}
usb_filesystem_label: '{{ usb_filesystem_label|default('V2M_MPS') }}'

actions:
  deploy:
    methods:
      lxc:
      mps:
  boot:
    connections:
      lxc:
      serial:
    methods:
      lxc:
      minimal:
{% endblock body -%}
