# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-06 15:22
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_results_app", "0013_buglinks")]

    operations = [
        migrations.AlterField(
            model_name="chartquery",
            name="xaxis_attribute",
            field=models.CharField(
                blank=True, max_length=100, null=True, verbose_name="X-axis attribute"
            ),
        )
    ]
