{# device_type = "sharkl2" #}
{% extends 'base-uboot.jinja2' %}
{% set bootloader_prompt = bootloader_prompt|default('=>') %}
{% set interrupt_prompt = interrupt_prompt|default('Hit any key to stop autoboot') %}
{% set interrupt_char = interrupt_char|default(' ') %}

{% set uboot_mkimage_arch = 'arm' %}
{% set adb_serial_number = adb_serial_number|default("'0000000000'") %}
{% set fastboot_serial_number = fastboot_serial_number|default("'0000000000'") %}
{% set device_info = device_info|default([{'board_id': fastboot_serial_number}]) %}
{% set console_device = console_device|default('ttyS2') %}
{% set baud_rate = baud_rate|default(115200) %}
{% set uboot_extra_error_message = 'Invalid partition' %}
{% set fastboot_sequence = [
    'reboot'] %}
{% set fastboot_deploy_uboot_commands = fastboot_deploy_uboot_commands|default(["cboot fastboot"]) %}
{% set fastboot_boot_uboot = " " %}
