{% extends "layouts/content.html" %}
{% load i18n %}

{% block title %}{{ block.super }}| {% trans "Password changed" %}{% endblock %}

{% block extrahead %}
  <meta http-equiv="refresh" content="5;url={% url 'lava.me' %}">
{% endblock %}

{% block content %}
<h2>{% trans "Your new password has been set" %}</h2>

<p>You will be redirected to the <a href="{% url 'lava.me' %}">profile</a> page in 5 seconds ...</p>

{% endblock %}
