<!DOCTYPE html>
<html lang="{{ LANGUAGE_CODE }}">
<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <title>{% block title %}{% endblock %}</title>

  <link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}lava_server/css/bootstrap-3.3.7.css">
  <link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}lava_server/css/jquery.typeahead.min.css"/>

  <script>
    // Hack for fixed navbar anchor problem.
    // Source: https://github.com/twbs/bootstrap/issues/1786
    var shiftWindow = function() { scrollBy(0, -50) };
    window.addEventListener("hashchange", shiftWindow);
    function fix_anchor_offset() { if (window.location.hash) shiftWindow(); }
  </script>

  <style>
    body {
      padding-top: 60px;
      margin-bottom: 100px;
    }

    html {
      position: relative;
      min-height: 100%;
    }
    #footer {
      position: absolute;
      bottom: 0;
      width: 100%;
      /* Set the fixed height of the footer here */
      height: 100px;
      background-color: #f5f5f5;
    }
    #footer > .container {
      padding-top: 15px;
      padding-right: 15px;
      padding-left: 15px;
    }
    colgroup col.sorted {
      background-color: #F8F8F8;
    }

    table.table thead {
      background-color: #F0F0F0;
    }
    .typeahead__result {
       display: inline;
       width: unset;
    }
    .typeahead__container {
      display: inline;
      font: unset;
    }
    .typeahead__field {
      display: inline;
    }
    .typeahead__field > * {
      vertical-align: unset;
    }

  </style>
  {% block styles %}{% endblock %}
  {% block extrahead %}{% endblock %}
</head>

<body onload="fix_anchor_offset()">
{% block body %}{% endblock %}

<script type="text/javascript" src="{{ STATIC_URL }}lava_server/js/jquery-1.11.0.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}lava_server/js/bootstrap-3.3.7.min.js"></script>
{% block scripts %}
{% endblock scripts %}
</body>
</html>
