{# set device_type = "b2120h410" #}
{% extends 'base-uboot.jinja2' %}

{% set console_device = console_device|default('ttyUSB0') %}
{% set baud_rate = baud_rate|default(115200) %}
{% if extra_kernel_args %}
{% set extra_kernel_args = "consoleblank=0 mem=1071644672@0x40000000 vmalloc=256m video=HDMI-A-1:1920x1080 snd-usb-audio.index=-2 earlyprintk consoleblank=0 " + extra_kernel_args %}
{% else %}
{% set extra_kernel_args = "consoleblank=0 mem=1071644672@0x40000000 vmalloc=256m video=HDMI-A-1:1920x1080 snd-usb-audio.index=-2 earlyprintk consoleblank=0" %}
{% endif %}
{% set bootloader_prompt = bootloader_prompt|default('B2120') %}
{% set boot_message = 'Starting kernel' %}
{% set bootm_kernel_addr = '0x41000000' %}
{% set bootm_ramdisk_addr = '0x45000000' %}
{% set bootm_dtb_addr = '0x47000000' %}
{% set base_ip_args = 'ip=::::stiH410::dhcp kmac=dev:eth0,addr:${ethaddr}' %}
{% set uboot_needs_usb = true %}
{% set uboot_mkimage_arch = 'arm' %}
