# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-06-24 08:49
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0019_extendeduser")]

    operations = [
        migrations.AlterUniqueTogether(
            name="notificationrecipient",
            unique_together=set([("user", "notification", "method")]),
        )
    ]
