# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-04-14 13:47
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dashboard_app', '0021_auto_20151207_0921'),
    ]

    operations = [
        migrations.AlterField(
            model_name='imagechartfilter',
            name='filter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='dashboard_app.TestRunFilter'),
        ),
    ]
