{% extends 'base-uboot.jinja2' %}

{% set booti_dtb_addr     = '0x01f00000' %}
{% set booti_kernel_addr  = '0x02080000' %}
{% set booti_ramdisk_addr = '0x04000000' %}

{% set uboot_mkimage_arch = 'arm64' %}
