# Peach pi AKA Samsung Chromebook 2 running u-boot (chainloaded by vendor
# u-boot)
{# device_type = "peach-pi" #}
{% extends 'exynos5420-common.jinja2' %}

{% set bootloader_prompt = bootloader_prompt|default('Peach') %}
{% set console_device = console_device|default('ttySAC3') %}
