{% extends "layouts/content.html" %}

{% block content %}
<h2>Annotate Job Failure - {{ job.id }} </h2>

{% if form.errors %}
<h3>Errors found in submission</h3>
{{ form.errors }}
{% endif %}

<form action="{% url 'lava.scheduler.job.annotate_failure' job.pk %}" method="post" class="form-horizontal">
  {% csrf_token %}
  <div class="form-group">
    <label for="id_failure_tags" class="col-sm-2 control-label">Tags</label>
    <div class="col-sm-10">
      {{ form.failure_tags }}
    </div>
  </div>
  <div class="form-group">
    <label for="id_failure_comment" class="col-sm-2 control-label">Comment</label>
    <div class="col-sm-10">
      {{ form.failure_comment }}
    </div>
  </div>
  <button type="submit" class="btn btn-success">Submit</button>
</form>
{% endblock %}
