{# device_type: nrf52-nitrogen #}
{% extends 'base.jinja2' %}
{% block body %}
board_id: '{{ board_id|default('0000000000') }}'

actions:
  deploy:
    methods:
      image:

  boot:
    connections:
      serial:
      ssh:
    methods:
      pyocd:
        parameters:
          command:
            pyocd-flashtool
          options:
          - -d {{ debug|default('debug') }}
          - -t nrf52
          - -f 3000000
{% endblock -%}
