{% extends "lava_scheduler_app/_device_base.html" %}
{% load django_tables2 %}
{% load i18n %}
{% load utils %}

{% block device_heading %}
<ul class="pager">
  <li class="previous{% if not previous_device %} disabled{% endif %}">
    <a href="{% if previous_device %}{% url 'lava.scheduler.device.detail' previous_device %}{% else %}#{% endif %}"><span
            class="glyphicon glyphicon-backward"></span> {% trans "Previous device" %}</a>
  </li>
  <li class="next{% if not next_device %} disabled{% endif %}">
    <a href="{% if next_device %}{% url 'lava.scheduler.device.detail' next_device %}{% else %}#{% endif %}">{% trans "Next Device" %} <span class="glyphicon glyphicon-forward"></span></a>
  </li>
</ul>
{% endblock %}

{% block content_columns %}
<hr/>
<h3>Recent jobs on {{ device.hostname }}</h3>
  {% render_table recent_job_table %}

<h3>Transitions</h3>
  {% render_table device_log_table %}

{% endblock %}
