{% extends "layouts/content.html" %}

{% load django_tables2 %}
{% load i18n %}

{% block content %}

<h2>Health History Log for {{ device_type.name }}</h2>

<div id="device_type_health_history_log">
{% render_table dthealthhistory_table %}
</div>

{% endblock %}

{% block scripts %}
<script type="text/javascript" src="{{ STATIC_URL }}lava_scheduler_app/js/tables.min.js"></script>
{% endblock %}
