{% extends "layouts/content.html" %}
{% load i18n %}

{% block title %}{% trans "Password reset" %}{% endblock %}


{% block content %}
<div class="row">
  <div class="col-md-12">
    <h2>{% trans "Password reset successful" %}</h2>
    <p>{% trans "We've emailed you instructions for setting your password to the email address you submitted. You should be receiving it shortly." %}</p>
  </div>
</div>
{% endblock %}
