{% extends 'base-grub.jinja2' %}
{% set boot_character_delay = boot_character_delay | default(200) %}
{% set efi_grub = True %}
{% set action_timeout_bootloader_interrupt = 300 %}
{% set efinet = True %}
{% set extra_nfsroot_args = ",vers=3" %}
{% set console_device = False %}
