# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-07-05 17:19
import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0020_notificationrecipient_uniquetogether")]

    operations = [
        migrations.RemoveField(model_name="notification", name="blacklist"),
        migrations.AddField(
            model_name="notification",
            name="blacklist",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.CharField(blank=True, max_length=100),
                blank=True,
                null=True,
            ),
        ),
    ]
