{# device_type = "arndale" #}
{% extends 'base-uboot.jinja2' %}

{% set console_device = console_device|default('ttySAC2') %}
{% set baud_rate = baud_rate|default(115200) %}
{% if extra_kernel_args %}
{% set extra_kernel_args = "drm_kms_helper.edid_firmware=edid-1920x1080.fw " + extra_kernel_args %}
{% else %}
{% set extra_kernel_args = "drm_kms_helper.edid_firmware=edid-1920x1080.fw" %}
{% endif %}
{% set bootloader_prompt = bootloader_prompt|default('ARNDALE5250') %}
{% set bootm_kernel_addr = '0x40007000' %}
{% set bootm_ramdisk_addr = '0x45000000' %}
{% set bootm_dtb_addr = '0x41f00000' %}
{% set bootz_kernel_addr = bootm_kernel_addr %}
{% set bootz_ramdisk_addr = bootm_ramdisk_addr %}
{% set bootz_dtb_addr = bootm_dtb_addr %}
{% set base_ip_args = 'ip=:::::eth0:dhcp' %}
{% set uboot_mkimage_arch = 'arm' %}
