# Tegra 124 Jetson TK1
{# device_type = "jetson-tk1" #}
{% extends 'tegra124-common.jinja2' %}

{% set extra_nfsroot_args = ",vers=3" %}
{% set bootloader_prompt = bootloader_prompt|default('Jetson TK1') %}

{% set base_ip_args = 'ip=:::::eth0:dhcp' %}
