{% extends 'hi6220-hikey.jinja2' %}
{% set adb_serial_number = '0123456789' %}
{% set fastboot_serial_number = 'usb:2-1.2' %}
{% set connection_list = ['uart0', 'uart1'] %}
{% set connection_commands = {'uart0': 'telnet localhost 4002', 'uart1': 'telnet 192.168.1.200 8001'} %}
{% set connection_tags = {'uart1': ['primary']} %}
{% set hard_reset_command = '/home/stylesen/work/pdu/pdu-control-reset.sh 0 5 1 5' %}
{% set soft_reset_command = 'fastboot -s usb:2-1.2 reboot' %}
{% set power_on_command = '/home/stylesen/work/pdu/pdu-control-on.sh 5' %}
{% set power_off_command = '/home/stylesen/work/pdu/pdu-control-off.sh 5' %}
{% set fastboot_options = ['-S', '256M'] %}
{% set device_info = [{'board_id': '0123456789'}] %}
{% set static_info = [{'board_id': 'S_N0123456'}] %}
