# Radxa Rock square, running upstream u-boot (not the vendor u-boot)
{% extends 'base-uboot.jinja2' %}

{% set device_type = "rk3288-rock2-square" %}

{% set console_device = console_device|default('ttyS2') %}

{% set bootm_kernel_addr = '0x2000000' %}
{% set bootm_ramdisk_addr = '0x4000000' %}
{% set bootm_dtb_addr = '0x1f00000' %}
{% set bootz_kernel_addr = '0x2000000' %}
{% set bootz_ramdisk_addr ='0x4000000' %}
{% set bootz_dtb_addr = '0x1f00000' %}
{% set uboot_mkimage_arch = 'arm' %}
