{# device_type = "beaglebone-black" #}
{% extends 'base-uboot.jinja2' %}

{% set console_device = console_device|default('ttyO0') %}
{% set baud_rate = baud_rate|default(115200) %}
{% set boot_character_delay = boot_character_delay | default(10) %}
{% set bootloader_prompt = bootloader_prompt|default('U-Boot') %}
{% set bootz_kernel_addr = '0x82000000' %}
{% set bootz_ramdisk_addr = '0x83000000' %}
{% set bootz_dtb_addr = '0x88000000' %}
{% set bootm_kernel_addr = bootz_kernel_addr %}
{% set bootm_ramdisk_addr = bootz_ramdisk_addr %}
{% set bootm_dtb_addr = bootz_dtb_addr %}
{% set usb_uuid = usb_uuid|default('') %}
{% set usb_device_id = usb_device_id|default('') %}
{% set uboot_mkimage_arch = 'arm' %}
