# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-07 09:21
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dashboard_app', '0020_auto_20151124_1419'),
    ]

    operations = [
        migrations.AlterOrderWithRespectTo(
            name='testresult',
            order_with_respect_to=None,
        ),
    ]
