{# set device_type = "b2260" #}
{% extends 'base-uboot.jinja2' %}

{% set console_device = console_device|default('ttyAS1') %}
{% set baud_rate = baud_rate|default(115200) %}
{% if extra_kernel_args %}
{% set extra_kernel_args = "consoleblank=0 mem=992M@0x40000000 vmalloc=256m clk_ignore_unused earlyprintk " + extra_kernel_args %}
{% else %}
{% set extra_kernel_args = "consoleblank=0 mem=992M@0x40000000 vmalloc=256m clk_ignore_unused earlyprintk" %}
{% endif %}
{% set bootloader_prompt = bootloader_prompt|default('=>') %}
{% set boot_message = 'Starting kernel' %}
{% set bootm_kernel_addr = '0x60000000' %}
{% set bootm_ramdisk_addr = '0x45000000' %}
{% set bootm_dtb_addr = '0x47000000' %}
{% set base_ip_args = 'ip=::::stiH410::dhcp kmac=dev:eth0,addr:${ethaddr}' %}
{% set uboot_mkimage_arch = 'arm' %}
{% set uboot_set_mac = true %}
{% set action_timeout_power_off = 15 %}
