{% extends 'x15-bl.jinja2' %}
{% set adb_serial_number = '1900d00f3b1400a2' %}
{% set device_info = [{'board_id': '1900d00f3b1400a2'}] %}
{% set fastboot_serial_number = '1900d00f3b1400a2' %}
{% set power_off_command = ['/usr/local/lab-scripts/snmp_pdu_control --hostname 192.168.0.33 --command off --port 1',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 1 -s on',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 2 -s on',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 3 -s on',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 4 -s on'] %}
{# We use the ethernet controlled relay to emulate the power button press #}
{% set hard_reset_command = ['/usr/local/lab-scripts/snmp_pdu_control --hostname 192.168.0.33 --command reboot --port 1 --delay 20',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 1 -s offon',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 2 -s offon',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 3 -s offon',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 4 -s onoff'] %}
{% set connection_list = ['uart0'] %}
{% set connection_commands = {'uart0': 'telnet localhost 7001'} %}
{% set connection_tags = {'uart0': ['primary', 'telnet']} %}
{% set power_on_command = ['/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 4 -s off',
'/usr/local/lab-scripts/snmp_pdu_control --hostname 192.168.0.33 --command on --port 1'] %}
{% set flash_cmds_order = ['update',
'ptable',
'partition',
'hyp',
'modem',
'rpm',
'sbl1',
'sbl2',
'sec',
'tz',
'aboot',
'boot',
'rootfs',
'vendor',
'system',
'cache',
'userdata'] %}
{% set recovery_mode_command = [
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 1 -s off',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 2 -s off',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 3 -s off',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 4 -s off'] %}
{% set recovery_exit_command = [
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 1 -s on',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 2 -s on',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 3 -s on',
'/usr/local/lab-scripts/eth008_control -a 192.168.0.32 -r 4 -s on'] %} 
