{% extends "layouts/content.html" %}
{% load django_tables2 %}

{% block content %}
<h2>Job Queue</h2>

<ul class="pager">
    <li class="btn btn-sm"><span class="glyphicon glyphicon-asterisk">
        <a href="{% url 'lava.scheduler.job.list' %}">All Jobs</a></span></li>
    <li class="btn btn-sm"><span class="glyphicon glyphicon-play">
        <a href="{% url 'lava.scheduler.job.active' %}">Active Jobs</a></span></li>
    <li class="btn btn-sm disabled"><span class="glyphicon glyphicon-pause">
        <a href="{% url 'lava.scheduler.queue' %}">Queued Jobs</a></span></li>
    <li class="btn btn-sm"><span class="glyphicon glyphicon-heart">
        <a href="{% url 'lava.scheduler.healthcheck' %}">Healthcheck Jobs</a></span></li>
    <li class="btn btn-sm"><span class="glyphicon glyphicon-flag">
        <a href="{% url 'lava.scheduler.job.errors' %}">Job Errors</a></span></li>
</ul>

<h3>TestJobs in state 'submitted'</h3>

{% render_table queue_table %}

{% endblock %}

{% block scripts %}
<script type="text/javascript" src="{{ STATIC_URL }}lava_scheduler_app/js/tables.min.js"></script>
{% endblock %}
