{% extends "layouts/content.html" %}
{% load i18n %}

{% block title %}{% trans "Password reset complete" %}{% endblock %}


{% block big_message %}{% trans "Success!" %}{% endblock %}

{% block content %}
<div class="row">
  <div class="col-md-12">
    <p>{% trans "Your password has been set.  You may go ahead and log in now." %}</p>
    <p><a class="btn btn-success" href="{{ login_url }}">{% trans 'Log in' %}</a></p>
  </div>
</div>
{% endblock %}
