{% extends 'qemu.jinja2' %}
{% set memory = 1024 %}
{% set mac_addr = '52:54:00:12:34:59' %}
{% set user_commands = {'touch': {'do': "touch /tmp/bla", 'undo': "touch /tmp/bla.rm"},
                        'sleep_short': {'do': "sleep 10"},
                        'sleep': {'do': "sleep 100", 'undo': "/bin/true"}} %}
