{# device_type = "sun6i-a31-app4-evb1" #}
{% extends 'sunxi-common.jinja2' %}

{% set uboot_mkimage_arch = 'arm' %}
{% set uboot_needs_usb = true %}
{% set uboot_usb_commands = (
"          - usb reset
          - setenv ethact 'asix_eth'") -%}
