{% extends "layouts/base.html" %}
{% load i18n %}
{% load utils %}

{% block title %}{{ bread_crumb_trail|join:" | " }}{% endblock %}


{% block body %}
  {% block header %}
  <!-- header -->
  {% block menu %}
  <!-- menu -->
  <nav class="navbar navbar-default navbar-fixed-top">
    <div class="container">
      <div class="navbar-header">
        <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar" aria-expanded="false" aria-controls="navbar">
          <span class="sr-only">Toggle navigation</span>
          <span class="icon-bar"></span>
          <span class="icon-bar"></span>
          <span class="icon-bar"></span>
        </button>
        {% if 'lava_server/images' in lava.branding_icon %}
        <a class="navbar-brand" href="{{ lava.branding_url }}"><p><img height={{ lava.branding_height }}
                width={{ lava.branding_width }} src="{{ STATIC_URL }}{{ lava.branding_icon }}" alt="{{ lava.branding_alt }}" /> LAVA</p></a>
        {% else %}
        <a class="navbar-brand" href="{{ lava.branding_url }}"><p><img height={{ lava.branding_height }}
                width={{ lava.branding_width }} src="{{ lava.branding_icon }}" alt="{{ lava.branding_alt }}" /> LAVA</p></a>
        {% endif %}
      </div>
      <div id="navbar" class="navbar-collapse collapse">
        {% include 'layouts/_navbar.html' %}
        {% include 'layouts/_userlinks.html' %}

      </div>
    </div>
  </nav>
  {% endblock menu %}
  {% endblock header %}

<!-- lava-container -->
<div class="container">
  {% block breadcrumb_container %}
    {% if context_help %}
       <span style="float: right; padding-right: 1em; padding-top: 0.5em"><a target="_blank"
           href="{{ STATIC_URL }}docs/v2/{{ context_help|join:'-'|slugify }}.html"
           ><b>Help</b></a></span>
    {% endif %}
  <ol class="breadcrumb">
    {% block breadcrumbs %}
    {% for bread_crumb in bread_crumb_trail %}
    <li><a href="{{ bread_crumb.get_absolute_url }}">{{ bread_crumb.get_name }}</a></li>
    {% endfor %}
    {% endblock breadcrumbs %}
  </ol>
  {% endblock breadcrumb_container %}

  {% block before_content %}
  {% endblock %}

  {% block content %}{% endblock %}
</div>

<div id="footer">
  <div class="container">
    <p class="text-center"><a
            href="{{ lava.branding_bug_url }}">Report a bug</a>&nbsp;&nbsp;|&nbsp;&nbsp;<a
            href="{{ STATIC_URL }}docs/v2/support.html">Support</a>&nbsp;&nbsp;|&nbsp;&nbsp;<a
            href="{{ lava.branding_source_url }}">Source code</a></p>
    <p class="text-muted text-center">LAVA is free software developed by <a
            href="http://linaro.org">Linaro</a> and the <a
            href="https://www.lavasoftware.org">LAVA Software Community Project</a>. It is distributed under the terms of the <a
            href="http://www.gnu.org/licenses/agpl-3.0.html" >GNU Affero General Public License version 3</a>.<br />
        <strong>You have the right to obtain source code of any server side installations of this software that you interact with.</strong></p>
  </div>
</div>
{% endblock body %}
