{% extends "admin/index.html" %}
{% load i18n %}

{% block bodyclass %}{{ block.super }} app-{{ app_label }}{% endblock %}

{% if not is_popup %}
{% block breadcrumbs %}
<div class="breadcrumbs">
[ <a href="{% url 'lava.home' %}">LAVA home</a> ] [ <a href="{% url 'lava.scheduler' %}">LAVA scheduler</a> ]
[ <a href="{% url 'lava_results' %}">LAVA results</a> ] [ <a href="{{ STATIC_URL }}docs/v2">LAVA Documentation</a> ]
<hr/>
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo;
{% for app in app_list %}
{{ app.name }}
{% endfor %}
</div>
{% endblock %}
{% endif %}

{% block sidebar %}{% endblock %}
