# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-08-01 10:43
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("lava_results_app", "0011_namedtestattribute")]

    operations = [
        migrations.AlterModelOptions(
            name="namedtestattribute", options={"verbose_name": "metadata"}
        )
    ]
