# Hardkernel Odroid X2
{# device_type = "odroid-x2" #}
{% extends 'exynos4-5-common.jinja2' %}

{% set bootloader_prompt = bootloader_prompt|default('Odroid') %}
{% set console_device = console_device|default('ttySAC1') %}
