{# device_type = "ifc6410" #}
{% extends 'base-fastboot.jinja2' %}
{% set fastboot_sequence = [
    'reboot', 'boot', 'auto-login', 'overlay-unpack', 'shell-session', 'export-env'] %}
{% set soft_reboot_command = '' %}
