{# device_type = "apq8016-sbc" #}
{% extends 'base-uboot.jinja2' %}

{% set console_device = console_device|default('ttyMSM0') %}
{% set baud_rate = baud_rate|default(115200) %}
{% if extra_kernel_args %}
{% set extra_kernel_args = "debug loglevel=7 " + extra_kernel_args %}
{% else %}
{% set extra_kernel_args = "debug loglevel=7" %}
{% endif %}
{% set bootloader_prompt = bootloader_prompt|default('=>') %}
{% set action_timeout_u_boot_commands = 90 %}
{% set booti_kernel_addr = '0x80080000' %}
{% set booti_kernel_addr = booti_kernel_addr|default('0x80080000') %}
{% set booti_ramdisk_addr = booti_ramdisk_addr|default('0x82e00000') %}
{% set booti_dtb_addr = booti_dtb_addr|default('0x82000000') %}
{% set bootm_kernel_addr = booti_kernel_addr %}
{% set bootm_ramdisk_addr = booti_ramdisk_addr %}
{% set bootm_dtb_addr = booti_dtb_addr %}
{% set uboot_initrd_high = '0xffffffffffffffff' %}
{% set uboot_fdt_high = '0xffffffffffffffff' %}
{% set uboot_needs_usb = true %}
{% set uboot_mkimage_arch = 'arm64' %}
{% set uboot_needs_interrupt = uboot_needs_interrupt|default(False) %}
{% set base_high_limits = False %}
