{% extends "layouts/content.html" %}
{% load i18n %}
{% load django_tables2 %}


{% block content %}
<h2>LAVA Results overview</h2>

{% render_table result_table %}

{% include "_bug_links.html" %}

{% endblock %}
{% block scripts %}
<script type="text/javascript">
  var content_type_id = '{{ content_type_id }}';
  var bug_links_url = "{% url 'lava.results.get_bug_links_json' %}";
  var delete_bug_url = "{% url 'lava.results.delete_bug_link' %}";
  var csrf_token = '{{csrf_token}}';
</script>
<script type="text/javascript" src="{{ STATIC_URL }}lava_scheduler_app/js/tables.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}lava_results_app/js/bug-links.min.js"></script>
{% endblock %}
