{% extends "base-uboot.jinja2" %}

{% set console_device = console_device|default("ttyS0") %}
{% set baud_rate = baud_rate|default(115200) %}

{% set base_high_limits = False %}
{% if extra_kernel_args %}
{% set extra_kernel_args = "rootwait coherent_pool=2M " + extra_kernel_args %}
{% else %}
{% set extra_kernel_args = "rootwait coherent_pool=2M" %}
{% endif %}

{% set uboot_needs_interrupt = uboot_needs_interrupt|default(False) %}
{% set bootloader_prompt = bootloader_prompt|default("=>") %}
{% set extra_nfsroot_args = ",vers=3" %}

{% set bootm_kernel_addr = "0x80008000" %}
{% set bootm_ramdisk_addr = "-" %}
{% set bootm_dtb_addr = "0x80f00000" %}

{% set uboot_mkimage_arch = "arm" %}
