{# device_type = "kirkwood-db-88f6282" #}
{% extends 'base-uboot.jinja2' %}
{% set console_device = console_device|default('ttyS0') %}
{% set baud_rate = baud_rate|default(115200) %}
{% set bootloader_prompt = bootloader_prompt|default('Marvell>>') %}
{% set bootm_kernel_addr = '0x02000000' %}
{% set bootm_ramdisk_addr = '0x03000000' %}
{% set bootm_dtb_addr = '0x01000000' %}
{% set base_ip_args = 'ip=dhcp' %}
{% set uboot_mkimage_arch = 'arm' %}
{% set uboot_bootx_cmd = "bootm {KERNEL_ADDR}" %}
{% set append_dtb = true %}
{% set use_xip = true %}
{% set uboot_tftp_commands = ("          - setenv loadkernel 'tftp {KERNEL_ADDR} {KERNEL}'
          - setenv loadinitrd ''
          - setenv loadfdt 'tftp {DTB_ADDR} {DTB}'") -%}
