# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-18 15:24
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0015_drop_heartbeat")]

    operations = [
        migrations.AlterField(
            model_name="testjob",
            name="submit_time",
            field=models.DateTimeField(
                auto_now_add=True, db_index=True, verbose_name="Submit time"
            ),
        )
    ]
