{% extends "layouts/content.html" %}

{% load django_tables2 %}

{% block content %}
<h2>Longest Running Jobs <a
class="btn btn-xs btn-info" href="{% url 'lava.scheduler.running' %}"
title="Running jobs and active devices"><span class="glyphicon glyphicon-time"></span> running jobs and active devices</a></h2>

{% render_table longestjobs_table %}

{% endblock %}

{% block scripts %}
<script type="text/javascript" src="{{ STATIC_URL }}lava_scheduler_app/js/tables.min.js"></script>
{% endblock %}
