# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-20 15:42
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [("lava_scheduler_app", "0033_remove_old_status_fields")]

    operations = [
        migrations.AddField(
            model_name="worker",
            name="last_ping",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Last ping"
            ),
        )
    ]
