{% extends "layouts/content.html" %}
{% load django_tables2 %}

{% block content %}
<h2>Lab Health</h2>

{% render_table device_health_table %}

{% endblock %}

{% block scripts %}
<script type="text/javascript" src="{{ STATIC_URL }}lava_scheduler_app/js/tables.min.js"></script>
{% endblock %}
