{# device_type: docker #}
{% extends 'base.jinja2' %}

{% block body %}

actions:
  deploy:
    methods:
      docker:
  boot:
    connections:
      serial:
    methods:
      docker:
        options:
          cpus: {{ docker_cpus|default(0.0) }}
          memory: {{ docker_memory|default(0) }}
{% if docker_devices|default([]) %}
          devices:
{% for device in docker_devices %}
            - {{ device }}
{% endfor %}
{% else %}
          devices: []
{% endif %}
{% if docker_volumes|default([]) %}
          volumes:
{% for volume in docker_volumes %}
            - {{ volume }}
{% endfor %}
{% else %}
          volumes: []
{% endif %}
{% endblock body %}
