{# device_type = 'mediatek-8173' #}
{% extends 'base-fastboot.jinja2' %}

{% set fastboot_sequence = [] %}
{% set flash_cmds_order = ['partition', 'lk', 'boot0', 'TEE1', 'TEE2',
                           'boot' ] %}
