# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-08-05 09:45
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("lava_results_app", "0012_namedtestattribute_metadata"),
    ]

    operations = [
        migrations.CreateModel(
            name="BugLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("url", models.URLField(max_length=1024, verbose_name="Bug Link URL")),
                ("object_id", models.PositiveIntegerField()),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        ),
        migrations.AlterUniqueTogether(
            name="buglink", unique_together=set([("object_id", "url", "content_type")])
        ),
    ]
