# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-04-13 16:10
import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("lava_results_app", "0008_auto_20160405_1543")]

    operations = [
        migrations.AddField(
            model_name="query",
            name="limit",
            field=models.PositiveIntegerField(
                default=200,
                validators=[django.core.validators.MinValueValidator(20)],
                verbose_name="Results limit",
            ),
        )
    ]
