<div class="text-nowrap">
  <a href="{{ record.get_absolute_url }}" class="btn btn-xs btn-primary" title="View test suite">
    <span class="glyphicon glyphicon-eye-open"></span>
  </a>
  <a href="{{ record.job.get_absolute_url }}" class="btn btn-xs btn-info" title="View job details">
    <span class="glyphicon glyphicon-chevron-left"></span>
  </a>
</div>

