# snow AKA Samsung arm Chromebook running u-boot (chainloaded by vendor
# u-boot)
{# device_type = "snow" #}
{% extends 'exynos4-5-common.jinja2' %}

{% set boot_character_delay = boot_character_delay | default(1) %}
{% set bootloader_prompt = bootloader_prompt|default('snow \#') %}
{% set console_device = console_device|default('ttySAC3') %}
