{# device_type = "imx6q-nitrogen6x" #}
{% extends 'base-uboot.jinja2' %}

{% set console_device = console_device|default('ttymxc1') %}
{% set baud_rate = baud_rate|default(115200) %}
{% set bootloader_prompt = bootloader_prompt|default('U-Boot >') %}
{% set bootm_kernel_addr = '0x11000000' %}
{% set bootm_ramdisk_addr = '0x13000000' %}
{% set bootm_dtb_addr = '0x12000000' %}
{% set bootz_kernel_addr = bootm_kernel_addr %}
{% set bootz_ramdisk_addr = bootm_ramdisk_addr %}
{% set bootz_dtb_addr = bootm_dtb_addr %}
{% set uboot_mkimage_arch = 'arm' %}

